/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/08/10 11:57:38
 */

import { Component } from 'wkit'
import {
  __STORE__,
  __ROUTER__,
  STORE_CALLBACKS,
  ROUTE_CALLBACKS
} from './constants.js'
import { noop } from './utils.js'

function observe(obj, paths = ['$store']) {
  if (obj === null) {
    return obj
  }

  return new Proxy(obj, {
    get(target, key, receiver) {
      let value = Reflect.get(target, key, receiver)
      // 当访问的值是对象时，需要对这个对象也进行代理
      if (typeof value === 'object') {
        return observe(value, paths.concat(key))
      }
      return value
    },
    set(target, key, value, receiver) {
      let full = paths.concat(key).join('.')
      if (target[key] === value) {
        return true
      }
      Reflect.set(target, key, value, receiver)
      if (STORE_CALLBACKS.get(full)) {
        STORE_CALLBACKS.get(full).forEach(callback => {
          callback(value)
        })
      }
      window.wkitd.broadcast()
      return true
    }
  })
}

export function watch(key, callback = noop) {
  if (key.startsWith('$store.')) {
    let list = STORE_CALLBACKS.get(key)
    if (list) {
      list.add(callback)
    } else {
      list = new Set()
      list.add(callback)
      STORE_CALLBACKS.set(key, list)
    }
  } else if (key.startsWith('$route')) {
    ROUTE_CALLBACKS.add(callback)
    callback(window.wkitd.get(__ROUTER__).route)
  } else {
    return console.error('watch() only work on $store and $route')
  }
}

export function createStore(obj = {}) {
  let defined = false

  return function () {
    Object.defineProperty(Component.prototype, '$store', {
      get() {
        window.wkitd.assign(this)
        return window.wkitd.get(__STORE__)
      },
      set(val) {
        if (defined) {
          return console.error(
            'Can not set readonly property $store of Component'
          )
        }
        window.wkitd.set(__STORE__, observe(val))
        defined = true
      }
    })
    Component.prototype.$store = obj
  }
}
